@@ -1,7 +1,7 @@
 /*
     SPDX-License-Identifier: GPL-2.0-or-later
     SPDX-FileCopyrightText: 2009-2012 Alexander Rieder <alexanderrieder@gmail.com>
-    SPDX-FileCopyrightText: 2017-2018 Alexander Semke (alexander.semke@web.de)
+    SPDX-FileCopyrightText: 2017-2022 Alexander Semke (alexander.semke@web.de)
 */
 
 #include "maximasession.h"
@@ -14,7 +14,6 @@
 #include "settings.h"
 
 #include <QDebug>
-#include <QProcess>
 #include <QTimer>
 #include <QStandardPaths>
 
@@ -35,9 +34,7 @@ const QRegularExpression MaximaSession::MaximaInputPrompt =
             QRegularExpression(QStringLiteral("(\\(\\s*%\\s*i\\s*[0-9\\s]*\\))"));
 
 
-MaximaSession::MaximaSession( Cantor::Backend* backend ) : Session(backend),
-    m_process(nullptr),
-    m_justRestarted(false)
+MaximaSession::MaximaSession( Cantor::Backend* backend ) : Session(backend)
 {
     setVariableModel(new MaximaVariableModel(this));
 }
@@ -59,8 +56,8 @@ void MaximaSession::login()
     m_process->start(MaximaSettings::self()->path().toLocalFile(), arguments);
     m_process->waitForStarted();
 
-    QString input;
     // Wait until first maxima prompt
+    QString input;
     while (!input.contains(QLatin1String("</cantor-prompt>")))
     {
         m_process->waitForReadyRead();
@@ -73,6 +70,11 @@ void MaximaSession::login()
     connect(m_process, SIGNAL(readyReadStandardError()), this, SLOT(readStdErr()));
     connect(m_process, SIGNAL(error(QProcess::ProcessError)), this, SLOT(reportProcessError(QProcess::ProcessError)));
 
+    //enable latex typesetting if needed
+    const QString& val = QLatin1String((isTypesettingEnabled() ? "t":"nil"));
+    evaluateExpression(QString::fromLatin1(":lisp(setf $display2d %1)").arg(val), Cantor::Expression::DeleteOnFinish, true);
+
+    //auto-run scripts
     if(!MaximaSettings::self()->autorunScripts().isEmpty()){
         QString autorunScripts = MaximaSettings::self()->autorunScripts().join(QLatin1String(";"));
         autorunScripts.append(QLatin1String(";kill(labels)")); // Reset labels after running autorun scripts
@@ -113,10 +115,19 @@ void MaximaSession::logout()
     qDebug()<<"logout done";
 }
 
+MaximaSession::Mode MaximaSession::mode() const {
+    return m_mode;
+}
+
+void MaximaSession::setMode(MaximaSession::Mode mode)
+{
+    m_mode = mode;
+}
+
 Cantor::Expression* MaximaSession::evaluateExpression(const QString& cmd, Cantor::Expression::FinishingBehavior behave, bool internal)
 {
     qDebug() << "evaluating: " << cmd;
-    MaximaExpression* expr = new MaximaExpression(this, internal);
+    auto* expr = new MaximaExpression(this, internal);
     expr->setFinishingBehavior(behave);
     expr->setCommand(cmd);
     expr->evaluate();
@@ -129,18 +140,18 @@ void MaximaSession::readStdErr()
    qDebug()<<"reading stdErr";
    if (!m_process)
        return;
-   QString out=QLatin1String(m_process->readAllStandardError());
+   QString out = QString::fromLocal8Bit(m_process->readAllStandardError());
 
    if(expressionQueue().size()>0)
    {
-       MaximaExpression* expr = static_cast<MaximaExpression*>(expressionQueue().first());
+       auto* expr = expressionQueue().first();
        expr->parseError(out);
    }
 }
 
 void MaximaSession::readStdOut()
 {
-    QString out = QLatin1String(m_process->readAllStandardOutput());
+    QString out = QString::fromLocal8Bit(m_process->readAllStandardOutput());
     m_cache += out;
 
     //collect the multi-line output until Maxima has finished the calculation and returns a new promt
@@ -155,7 +166,7 @@ void MaximaSession::readStdOut()
         return;
     }
 
-    MaximaExpression* expr = static_cast<MaximaExpression*>(expressionQueue().first());
+    auto* expr = expressionQueue().first();
     if (!expr)
         return; //should never happen
 
@@ -176,7 +187,7 @@ void MaximaSession::reportProcessError(QProcess::ProcessError e)
 
 void MaximaSession::currentExpressionChangedStatus(Cantor::Expression::Status status)
 {
-    Cantor::Expression* expression = expressionQueue().first();
+    auto* expression = expressionQueue().first();
     qDebug() << "expression status changed: command = " << expression->command() << ", status = " << status;
 
     switch (status)
@@ -184,12 +195,9 @@ void MaximaSession::currentExpressionChangedStatus(Cantor::Expression::Status st
     case Cantor::Expression::Done:
     case Cantor::Expression::Error:
         qDebug()<<"################################## EXPRESSION END ###############################################";
-        disconnect(expression, SIGNAL(statusChanged(Cantor::Expression::Status)),
-            this, SLOT(currentExpressionChangedStatus(Cantor::Expression::Status)));
-
+        disconnect(expression, &Cantor::Expression::statusChanged, this, &MaximaSession::currentExpressionChangedStatus);
         finishFirstExpression();
         break;
-
     default:
         break;
     }
@@ -203,15 +211,15 @@ void MaximaSession::runFirstExpression()
 
     if(!expressionQueue().isEmpty())
     {
-        MaximaExpression* expr = static_cast<MaximaExpression*>(expressionQueue().first());
-        QString command=expr->internalCommand();
-        connect(expr, SIGNAL(statusChanged(Cantor::Expression::Status)), this, SLOT(currentExpressionChangedStatus(Cantor::Expression::Status)));
+        auto* expr = expressionQueue().first();
+        const auto& command = expr->internalCommand();
+        connect(expr, &Cantor::Expression::statusChanged, this, &MaximaSession::currentExpressionChangedStatus);
 
         expr->setStatus(Cantor::Expression::Computing);
         if(command.isEmpty())
         {
             qDebug()<<"empty command";
-            expr->forceDone();
+            static_cast<MaximaExpression*>(expr)->forceDone();
         }
         else
         {
@@ -229,14 +237,15 @@ void MaximaSession::interrupt()
         if(m_process && m_process->state() != QProcess::NotRunning)
         {
 #ifndef Q_OS_WIN
-            const int pid=m_process->pid();
+            const int pid = m_process->processId();
             kill(pid, SIGINT);
 #else
             ; //TODO: interrupt the process on windows
 #endif
         }
-       foreach (Cantor::Expression* expression, expressionQueue())
+       for (auto* expression : expressionQueue())
             expression->setStatus(Cantor::Expression::Interrupted);
+
         expressionQueue().clear();
 
         qDebug()<<"done interrupting";
@@ -253,7 +262,7 @@ void MaximaSession::sendInputToProcess(const QString& input)
 
 void MaximaSession::restartMaxima()
 {
-    qDebug()<<"restarting maxima cooldown: "<<m_justRestarted;
+    qDebug()<<"restarting maxima cooldown: " << m_justRestarted;
 
     if(!m_justRestarted)
     {
@@ -283,10 +292,13 @@ void MaximaSession::restartsCooledDown()
 
 void MaximaSession::setTypesettingEnabled(bool enable)
 {
-    //we use the lisp command to set the variable, as those commands
-    //don't mess with the labels and history
-    const QString& val=QLatin1String((enable==true ? "t":"nil"));
-    evaluateExpression(QString::fromLatin1(":lisp(setf $display2d %1)").arg(val), Cantor::Expression::DeleteOnFinish, true);
+    if (m_process)
+    {
+        //we use the lisp command to set the variable, as those commands
+        //don't mess with the labels and history
+        const QString& val = QLatin1String((enable ? "t":"nil"));
+        evaluateExpression(QString::fromLatin1(":lisp(setf $display2d %1)").arg(val), Cantor::Expression::DeleteOnFinish, true);
+    }
 
     Cantor::Session::setTypesettingEnabled(enable);
 }
